"""Backup books from macOS Books to usable ePubs."""

import shutil
from argparse import Namespace
from pathlib import Path

from termcolor import cprint

from bochord.epub_dir import backup_epub_dir, get_dest_file_mtime


def backup_file(filename: Path, args: Namespace) -> bool:
    """Backup documents that aren't epub directories."""
    src_path = args.source / filename.name
    src_file_mtime = src_path.stat().st_mtime
    dest_path = args.dest / filename.name
    dest_file_mtime = get_dest_file_mtime(dest_path)
    if args.force or src_file_mtime > dest_file_mtime:
        cprint(f"\nArchiving: {filename}", "cyan")
        shutil.copy2(src_path, dest_path)
        return True
    return False


def prune(args: Namespace) -> None:
    """Prune docs from destination that aren't in the source."""
    dest_set = set(args.dest.iterdir())
    src_set = set(args.source.iterdir())
    extra_set = sorted(dest_set - src_set)
    if args.verbose:
        cprint(f"Removing: {extra_set}", "yellow")
    for filename in extra_set:
        filename.unlink()
        cprint("\tRemoved: {filename}", "yellow")


def run(args: Namespace) -> None:
    """Backup everything."""
    for filename in sorted(args.source.iterdir()):
        backup_func = (
            backup_epub_dir
            if filename.suffix == ".epub" and filename.is_dir()
            else backup_file
        )
        if not backup_func(filename, args):
            if args.verbose:
                cprint(f"\nNot updated: {filename}", "green")
            else:
                cprint(".", "green", end="")

    if not args.verbose:
        cprint("")

    if args.prune:
        prune(args)



#!/usr/bin/env python3
"""Bochord cli arg parsing."""

import argparse
from importlib.metadata import PackageNotFoundError, version
from pathlib import Path

from bochord.bochord import run

ICLOUD_BOOK_DIR = "Library/Mobile Documents/iCloud~com~apple~iBooks/Documents"
PROGRAM_NAME = "bochord"

try:
    __version__ = version(PROGRAM_NAME)
except PackageNotFoundError:
    __version__ = "test"


def get_arguments():
    """Get arguments with argparser."""
    usage = "%(prog)s [options] <backup_path>"
    desc = "Backup books from macOS Books to usable ePubs"
    source_default = Path.home() / ICLOUD_BOOK_DIR
    parser = argparse.ArgumentParser(usage=usage, description=desc)
    parser.add_argument(
        "-f",
        "--force",
        action="store_true",
        dest="force",
        default=0,
        help="force re-archive even if no file updates",
    )
    parser.add_argument(
        "-v",
        "--verbose",
        action="store_true",
        dest="verbose",
        default=0,
        help="be verbose",
    )
    parser.add_argument(
        "-V", "--version", action="version", version=f"%(prog)s {__version__}"
    )
    parser.add_argument(
        "-p",
        "--prune",
        action="store_true",
        dest="prune",
        default=0,
        help="prune documents from destination if missing from source.",
    )
    parser.add_argument(
        "-s",
        "--source",
        action="store",
        dest="source",
        type=Path,
        default=source_default,
        help="source directory (default: %(default)s)",
    )
    parser.add_argument(
        "dest",
        metavar="backup_path",
        type=Path,
        help="backup destination path",
        default=None,
    )

    return parser.parse_args()


def main():
    """Get cli arguments and run bochord."""
    args = get_arguments()
    run(args)



"""Backup iCloud iBooks epub dir as an epub archive."""
# iCloud stores epubs exploded on disk.

import os
from argparse import Namespace
from pathlib import Path
from zipfile import ZIP_DEFLATED, ZIP_STORED, ZipFile

from termcolor import cprint

ZIP_MTIME_MIN = 315644400.0  # 1 day after 1980 for timezones


def _get_src_file_mtime(src_file_path: Path) -> float:
    """Get source file mtime, but alter it if pkzip will reject it."""
    src_file_mtime = src_file_path.stat().st_mtime
    if src_file_mtime < ZIP_MTIME_MIN:
        cprint(f"\tUpdating mtime for zip compatibility: {src_file_path}", "yellow")
        src_file_path.touch()
        src_file_mtime = src_file_path.stat().st_mtime
    return src_file_mtime


def get_dest_file_mtime(dest_path: Path) -> float:
    """Get Destination file mtime."""
    return dest_path.stat().st_mtime if dest_path.exists() else 0.0


def _get_updated_files(
    archive_mtime: float, epub_dir_src_path: Path, args: Namespace
) -> frozenset[Path]:
    """Check for updated files."""
    src_paths = set()
    update = False

    for root, _, src_files in os.walk(epub_dir_src_path):
        root_path = Path(root)
        for src_filename in sorted(src_files):
            src_file_path = root_path / src_filename
            src_paths.add(src_file_path)
            src_file_mtime = _get_src_file_mtime(src_file_path)
            update = update or src_file_mtime > archive_mtime

    if not update and not args.force:
        src_paths = set()

    return frozenset(src_paths)


def _archive_epub(
    epub_dest_path: Path, src_paths: frozenset[Path], args: Namespace
) -> None:
    """Make a new archive in a tempfile."""
    cprint(f"\nArchiving: {epub_dest_path.name}", "cyan")
    new_epub_path = epub_dest_path.with_suffix(".epub_new")

    with ZipFile(
        new_epub_path, mode="w", compression=ZIP_DEFLATED, compresslevel=9
    ) as epub:
        for src_file_path in sorted(src_paths):
            ctype = ZIP_STORED if src_file_path.name == "mimetype" else ZIP_DEFLATED
            epub.write(src_file_path, compress_type=ctype)
            if args.verbose:
                cprint(f"\t{src_file_path}", "cyan")

    # Move tempfile over old epub
    new_epub_path.replace(epub_dest_path)


def backup_epub_dir(epub_dir_src_path: Path, args: Namespace) -> bool:
    """Compress the exploded epub dir to the backup destination."""
    epub_dest_path = args.dest / epub_dir_src_path.name
    archive_mtime = get_dest_file_mtime(epub_dest_path)
    if src_paths := _get_updated_files(archive_mtime, epub_dir_src_path, args):
        _archive_epub(epub_dest_path, src_paths, args)
        return True
    return False
